@@ -45,6 +45,9 @@ lavData <- function(data = NULL, # data.frame
     level.label <- character(0L)
   }
 
+  # allow empty categories of ordinal variable
+  allow.empty.cell <- lavoptions$allow.empty.cell
+
   # block.labels
   block.label <- character(0L)
   if (length(group.label) > 0L && length(level.label) == 0L) {
@@ -65,10 +68,14 @@ lavData <- function(data = NULL, # data.frame
     std.ov <- FALSE
   }
 
-  # missing?
-  missing <- lavoptions$missing
+  # missing? (lavCor() does not parse options before calling lavdata...)
+  missing <- tolower(lavoptions$missing)
   if (is.null(missing) || missing == "default") {
     missing <- "listwise"
+  } else if (missing %in% c("ml", "fiml", "direct")) {
+    missing <- "ml"
+  } else if (missing %in% c("ml.x", "fiml.x", "direct.x")) {
+    missing <- "ml.x"
   }
 
   # warn?
@@ -111,7 +118,7 @@ lavData <- function(data = NULL, # data.frame
         data <- as.data.frame(data, stringsAsFactors = FALSE)
       } else {
         lav_msg_stop(gettextf(
-          "data= argument is not a data.frame, but of class ",
+          "data= argument is not a data.frame, but of class %s",
           sQuote(class(data))))
       }
     }
@@ -147,7 +154,8 @@ lavData <- function(data = NULL, # data.frame
       ov.names.l = ov.names.l,
       std.ov = std.ov,
       missing = missing,
-      allow.single.case = allow.single.case
+      allow.single.case = allow.single.case,
+      allow.empty.cell = allow.empty.cell
     )
     sample.cov <- NULL # not needed, but just in case
   }
@@ -261,7 +269,7 @@ lavData <- function(data = NULL, # data.frame
     }
 
     # if std.ov = TRUE, give a warning (suggested by Peter Westfall)
-    if (std.ov) {
+    if (std.ov && !lavoptions$correlation) {
       lav_msg_warn(gettext(
         "std.ov argument is ignored if only sample statistics are provided."))
     }
@@ -504,7 +512,8 @@ lav_data_full <- function(data = NULL, # data.frame
                           ov.names.l = list(), # var per level
                           std.ov = FALSE, # standardize ov's?
                           missing = "listwise", # remove missings?
-                          allow.single.case = FALSE # allow single case?
+                          allow.single.case = FALSE, # allow single case?
+                          allow.empty.cell = FALSE
 ) {
   # number of groups and group labels
   if (!is.null(group) && length(group) > 0L) {
@@ -552,6 +561,9 @@ lav_data_full <- function(data = NULL, # data.frame
     group <- character(0L)
   }
 
+  # ensure allow.empty.cell is logical
+  if (is.null(allow.empty.cell)) allow.empty.cell <- FALSE
+
   # sampling weights
   if (!is.null(sampling.weights)) {
     if (is.character(sampling.weights)) {
@@ -708,7 +720,7 @@ lav_data_full <- function(data = NULL, # data.frame
   ov <- lav_dataframe_vartable(
     frame = data, ov.names = ov.names,
     ov.names.x = ov.names.x, ordered = ordered,
-    as.data.frame. = FALSE
+    as.data.frame. = FALSE, allow.empty.cell = allow.empty.cell
   )
 
   # do some checking
@@ -814,6 +826,26 @@ lav_data_full <- function(data = NULL, # data.frame
     lav_msg_warn(gettext(
       "all observed variables are exogenous; model may not be identified"))
   }
+  # check for perfect correlations (NOT including exo variables)
+  if (!allow.single.case && any(ov$type == "numeric")) {
+    num.idx <- which(ov$type == "numeric" & ov$exo == 0L)
+    COR <- try(cor(data[,ov$idx[num.idx]], use = "pairwise.complete.obs"),
+               silent = TRUE)
+    # replace any NAs by 0 (as we only wish to detect perfect correlations)
+    COR[is.na(COR)] <- 0
+    if (!inherits(COR, "try-error") &&
+        any(lav_matrix_vech(COR, diagonal = FALSE) == 1)) {
+      COR[upper.tri(COR, diag = TRUE)] <- 0
+      idx <- which(COR == 1)
+      this.names <- ov$name[num.idx]
+      bad.names <- this.names[ sort(unique(c(row(COR)[idx], col(COR)[idx]))) ]
+      # should we make this a hard stop? things will most likely fail later...
+      lav_msg_warn(gettextf(
+      "some observed variables are perfectly correlated;
+       please check your data; variables involved are: %s",
+       paste(bad.names, collapse = " ")))
+    }
+  }
 
   # prepare empty lists
 
@@ -829,6 +861,7 @@ lav_data_full <- function(data = NULL, # data.frame
   weights <- vector("list", length = ngroups)
 
   # collect information per upper-level group
+  # datam <- data.matrix(data) # YR: not yet, this breaks the stuart package!
   for (g in 1:ngroups) {
     # extract variables in correct order
     if (nlevels > 1L) {
@@ -849,6 +882,13 @@ lav_data_full <- function(data = NULL, # data.frame
           complete.cases(data[all.idx]))
         nobs[[g]] <- length(case.idx[[g]])
         norig[[g]] <- length(which(data[[group]] == group.label[g]))
+        # check for empty data
+        if (nobs[[g]] == 0L) {
+          lav_msg_stop(gettextf("all observations were deleted due to missing
+                                 data after listwise deletion in group [%s];
+                                 check your data
+                                 or consider a different option for the missing=                                 argument.", group.label[g]))
+        }
         # } else if(missing == "pairwise" && length(exo.idx) > 0L) {
         #    case.idx[[g]] <- which(data[[group]] == group.label[g] &
         #                           complete.cases(data[exo.idx]))
@@ -874,6 +914,12 @@ lav_data_full <- function(data = NULL, # data.frame
         case.idx[[g]] <- which(complete.cases(data[all.idx]))
         nobs[[g]] <- length(case.idx[[g]])
         norig[[g]] <- nrow(data)
+        if (nobs[[g]] == 0L) {
+          lav_msg_stop(gettext("all observations were deleted due to missing
+                                data after listwise deletion;
+                                check your data
+                                or consider a different option for the missing=                                 argument."))
+        }
         # } else if(missing == "pairwise" && length(exo.idx) > 0L) {
         #    case.idx[[g]] <- which(complete.cases(data[exo.idx]))
         #    nobs[[g]] <- length(case.idx[[g]])
@@ -895,6 +941,7 @@ lav_data_full <- function(data = NULL, # data.frame
     }
 
     # extract data
+    #X[[g]] <- datam[case.idx[[g]], ov.idx, drop = FALSE]
     X[[g]] <- data.matrix(data[case.idx[[g]], ov.idx, drop = FALSE])
     dimnames(X[[g]]) <- NULL ### copy?
 
@@ -924,7 +971,7 @@ lav_data_full <- function(data = NULL, # data.frame
     if (length(user.ordered.idx) > 0L) {
       for (i in user.ordered.idx) {
         X[[g]][, i][is.na(X[[g]][, i])] <- NA # change NaN to NA
-        X[[g]][, i] <- as.numeric(as.factor(X[[g]][, i]))
+        if (!allow.empty.cell) X[[g]][, i] <- as.numeric(as.factor(X[[g]][, i]))
         # possible alternative to the previous two lines:
         # X[[g]][,i] <- as.numeric(factor(X[[g]][,i], exclude = c(NA, NaN)))
       }
@@ -970,7 +1017,7 @@ lav_data_full <- function(data = NULL, # data.frame
     }
     # check variances per group (if we have multiple groups)
     # to catch zero-variance variables within a group (new in 0.6-8)
-    if (ngroups > 1L) {
+    if (ngroups > 1L && !allow.empty.cell) {
       # X
       group.var <- apply(X[[g]], 2, var, na.rm = TRUE)
       zero.var <- which(group.var < .Machine$double.eps)
@@ -1152,6 +1199,11 @@ lav_data_full <- function(data = NULL, # data.frame
 	    small.idx <- which(coverage.vech < 0.1)
 	    if (all(coverage.vech[small.idx] == 0)) {
 		  # 0.6-18: no warning --> this could be due to missing by design
+          # 0.6-20: give warning anyway (as EM is ignoring this)
+          lav_msg_warn(gettext(
+            "due to missing values, some pairwise combinations have zero
+             coverage; the corresponding covariances are not identified;
+             use lavInspect(fit, \"coverage\") to investigate."))
 		} else {
           lav_msg_warn(gettext(
             "due to missing values, some pairwise combinations have less than
