@@ -69,6 +69,7 @@ lav_model_get_parameters <- function(lavmodel = NULL, GLIST = NULL,
 # warning: if categorical/correlation: 'delta' parameterization does
 #          not work properly if we have 'mediators' (where x is not fixed)
 #          that are observed (residuals are in PSI, and are not additive)
+#          Note: fixed in 0.6-20 for recursive models
 lav_model_set_parameters <- function(lavmodel = NULL, x = NULL) {
   tmp <- lavmodel@GLIST
   for (mm in 1:length(lavmodel@GLIST)) {
@@ -124,6 +125,21 @@ lav_model_set_parameters <- function(lavmodel = NULL, x = NULL) {
     }
   }
 
+  if (.hasSlot(lavmodel, "composites") && lavmodel@composites) {
+    nmat <- lavmodel@nmat
+    if (lavmodel@representation == "LISREL") {
+      for (g in 1:lavmodel@nblocks) {
+        # which mm belong to group g?
+        mm.in.group <- 1:nmat[g] + cumsum(c(0L, nmat))[g]
+
+        tmp[mm.in.group] <-
+          setVarianceComposites.LISREL(MLIST = tmp[mm.in.group])
+      }
+    } else {
+      cat("FIXME: deal with Composites if representation = RAM")
+    }
+  }
+
   lavmodel@GLIST <- tmp
 
   lavmodel
@@ -140,6 +156,7 @@ lav_model_x2GLIST <- function(lavmodel = NULL, x = NULL,
     correlation <- FALSE
   }
 
+
   GLIST <- lavmodel@GLIST
   for (mm in 1:length(GLIST)) {
     # skip empty matrix
@@ -219,6 +236,21 @@ lav_model_x2GLIST <- function(lavmodel = NULL, x = NULL,
     }
   }
 
+  if (.hasSlot(lavmodel, "composites") && lavmodel@composites) {
+    nmat <- lavmodel@nmat
+    if (lavmodel@representation == "LISREL") {
+      for (g in 1:lavmodel@nblocks) {
+        # which mm belong to group g?
+        mm.in.group <- 1:nmat[g] + cumsum(c(0L, nmat))[g]
+
+        GLIST[mm.in.group] <-
+          setVarianceComposites.LISREL(MLIST = GLIST[mm.in.group])
+      }
+    } else {
+      cat("FIXME: deal with Composites when representation = RAM")
+    }
+  }
+
   GLIST
 }
 
