@@ -103,6 +103,11 @@ lav_object_independence <- function(object = NULL,
   } else {
     # 0.6-18: slower, but safer to just keep it
 
+    # 0.6-20 -- except if se = "bootstrap" -> "none"
+    if (lavoptions$se == "bootstrap") {
+      lavoptions$se <- "none"
+    }
+
     ## FIXME: if test = scaled, we need it anyway?
     # if(lavoptions$missing %in% c("two.stage", "two.stage.robust")) {
     # don't touch it
@@ -121,6 +126,7 @@ lav_object_independence <- function(object = NULL,
   lavoptions$check.post <- FALSE
   lavoptions$check.vcov <- FALSE
   lavoptions$optim.bounds <- list() # we already have the bounds
+  lavoptions$start <- "default" # don't re-use user-specified starting values
   lavoptions$rstarts <- 0L # no random starts
 
   # ALWAYS do.fit and set optim.method = "nlminb" (if npar > 0)
@@ -365,6 +371,10 @@ lav_object_extended <- function(object, add = NULL,
     )
   }
 
+  # switch off 'consider switching to parameterization = theta' warning
+  # (modindices!)
+  lavoptions$check.delta.cat.mediator <- FALSE
+
   FIT <- lavaan(LIST,
     slotOptions     = lavoptions,
     slotSampleStats = object@SampleStats,
@@ -389,18 +399,29 @@ lav_object_catml <- function(lavobject = NULL) {
 
   # if only categorical variables: remove thresholds and intercepts
   refit <- FALSE
-  if (all(lavdata@ov$type == "ordered")) {
-    partable.catml <- parTable(lavobject)
-    rm.idx <- which(partable.catml$op %in% c("|", "~1"))
-    partable.catml <- partable.catml[-rm.idx, ]
-    partable.catml <- lav_partable_complete(partable.catml)
-  } else {
+  partable.catml <- parTable(lavobject)
+  rm.idx <- which(partable.catml$op %in% c("|", "~1"))
+  # remove also any constraints that refer to the labels that belong
+  # to these intercepts/thresholds (new in 0.6-20)
+  th.int.labels <- unique(partable.catml$label[rm.idx],
+                          partable.catml$plabel[rm.idx])
+  if (any(nchar(th.int.labels) == 0L)) {
+    th.int.labels <- th.int.labels[- which(nchar(th.int.labels) == 0L)]
+  }
+  con.idx <- which(partable.catml$op %in% c("==", "<", ">", ":=") &
+                   (partable.catml$lhs %in% th.int.labels |
+                    partable.catml$rhs %in% th.int.labels))
+  partable.catml <- partable.catml[-c(rm.idx, con.idx), ]
+  # never rotate (new in 0.6-19), as we only need fit measures
+  if (!is.null(partable.catml$efa)) {
+    partable.catml$efa <- NULL
+    partable.catml$free <- partable.catml$free.unrotated
+  }
+
+  if (!all(lavdata@ov$type == "ordered")) {
     refit <- TRUE
-    partable.catml <- parTable(lavobject)
     partable.catml$start <- partable.catml$est
     partable.catml$se <- NULL
-    rm.idx <- which(partable.catml$op %in% c("|", "~1"))
-    partable.catml <- partable.catml[-rm.idx, ]
     partable.catml$ustart <- partable.catml$est
     for (b in seq_len(lavpta$nblocks)) {
       ov.names.num <- lavpta$vnames$ov.num[[b]]
@@ -409,8 +430,8 @@ lav_object_catml <- function(lavobject = NULL) {
         partable.catml$lhs == partable.catml$rhs)
       partable.catml$free[ov.var.idx] <- 0L
     }
-    partable.catml <- lav_partable_complete(partable.catml)
   }
+  partable.catml <- lav_partable_complete(partable.catml)
 
   # adapt lavsamplestats
   for (g in seq_len(lavdata@ngroups)) {
@@ -463,7 +484,8 @@ lav_object_catml <- function(lavobject = NULL) {
   lavoptions$h1.information <- c("structured", "structured") # unlike DWLS
   lavoptions$se <- "none"
   lavoptions$test <- "standard" # always for now
-  lavoptions$baseline <- TRUE
+  lavoptions$rotation <- "none" # new in 0.6-19
+  lavoptions$baseline <- TRUE # also for RMSEA?
   if (!refit) {
     lavoptions$optim.method <- "none"
     lavoptions$optim.force.converged <- TRUE
