@@ -213,7 +213,11 @@ lav_partable_indep_or_unrestricted <- function(lavobject = NULL,
 
       # ov.names for this block
       if (is.null(lavpta)) { # only data was used
-        ov.names <- lavdata@ov.names[[g]]
+	    if (nlevels > 1L) {
+		  ov.names <- lavdata@ov.names.l[[g]][[(ngroups - 1)*g + b]]
+		} else {
+          ov.names <- lavdata@ov.names[[g]]
+		}
         ov.names.x <- lavdata@ov.names.x[[g]]
         ov.names.nox <- ov.names[!ov.names %in% ov.names.x]
       } else {
@@ -318,7 +322,7 @@ lav_partable_indep_or_unrestricted <- function(lavobject = NULL,
           # check for 'missing by design' cells: here, the sample.cov
           # element is *exactly* zero (new in 0.6-18)
           zero.cov <- which(sample.cov.vech == 0)
-          if (length(zero.cov) > 0L) {
+          if (length(zero.cov) > 0L && !is.null(lavh1)) {
             n.tmp <- length(free)
             ones.and.zeroes <- rep(1L, pstar)
             ones.and.zeroes[zero.cov] <- 0L
@@ -669,8 +673,9 @@ lav_partable_indep_or_unrestricted <- function(lavobject = NULL,
 lav_partable_unrestricted_chol <- function(lavobject = NULL,
                                            # if no object is available,
                                            lavdata = NULL,
-                                           lavpta = NULL,
-                                           lavoptions = NULL) {
+                                           lavpta = NULL, # optional
+                                           lavoptions = NULL,
+                                           group = NULL) {
   # grab everything from lavaan lavobject
   if (!is.null(lavobject)) {
     stopifnot(inherits(lavobject, "lavaan"))
@@ -691,6 +696,17 @@ lav_partable_unrestricted_chol <- function(lavobject = NULL,
   ngroups <- lavdata@ngroups
   nlevels <- lavdata@nlevels
 
+  # select groups
+  if (is.null(group)) {
+    group.select <- seq_len(ngroups)
+  } else {
+    stopifnot(is.numeric(group), all(group <= ngroups))
+    group.select <- group
+    if (length(group.select) == 0L) {
+      lav_msg_stop(gettext("no groups selected"))
+    }
+  }
+
   # what with fixed.x?
   # - does not really matter; fit will be saturated anyway
   # - fixed.x = TRUE may avoid convergence issues with non-numeric
@@ -711,6 +727,12 @@ lav_partable_unrestricted_chol <- function(lavobject = NULL,
   # block number
   b <- 0L
   for (g in 1:ngroups) {
+
+    # select group?
+    if (! g %in% group.select) {
+      next
+    }
+
     # only for multilevel
     if (nlevels > 1L) {
       Lp <- lavdata@Lp[[g]]
