@@ -36,6 +36,7 @@ lavaanNames <- lavNames # nolint
 #   or returns a list per block
 #   LDW 30/1/24: 'block' argument not explicitly tested !?
 #   LDW 29/2/24: ov.order = "data" via attribute "ovda"
+# - YR 11/02/25: add type = "lv.ho" for higher-order latent variables
 lav_partable_vnames <- function(partable, type = NULL, ..., # nolint
                                 force.warn = FALSE, ov.x.fatal = FALSE) {
   # This function derives the names of some types of variable (as specified
@@ -77,7 +78,8 @@ lav_partable_vnames <- function(partable, type = NULL, ..., # nolint
     "ov.y", # (pure) endogenous variables (dependent only)
     "ov.num", # numeric observed variables
     "ov.ord", # ordinal observed variables
-    "ov.ind", # observed indicators of latent variables
+    "ov.ind",  # observed indicators of latent variables
+    "ov.cind", # observed indicators of composites (new in 0.6-20)
     "ov.orphan", # lonely observed intercepts/variances
     "ov.interaction", # interaction terms (with colon)
     "ov.efa", # indicators involved in efa
@@ -87,7 +89,8 @@ lav_partable_vnames <- function(partable, type = NULL, ..., # nolint
 
     "lv", # latent variables
     "lv.regular", # latent variables (defined by =~ only)
-    "lv.formative", # latent variables (defined by <~ only)
+    "lv.formative", # latent variables (defined by <~ only) (old style)
+    "lv.composite", # latent variables (defined by <~ only) (new style)
     "lv.x", # (pure) exogenous variables
     "lv.y", # (pure) endogenous variables
     "lv.nox", # non-exogenous latent variables
@@ -96,6 +99,7 @@ lav_partable_vnames <- function(partable, type = NULL, ..., # nolint
     "lv.efa", # latent variables involved in efa
     "lv.rv", # random slopes, random variables
     "lv.ind", # latent indicators (higher-order cfa)
+    "lv.ho",  # higher-order latent variables
     "lv.marker", # marker indicator per lv
 
     "eqs.y", # y's in regression
@@ -244,6 +248,7 @@ lav_partable_vnames <- function(partable, type = NULL, ..., # nolint
         if ("lv.regular" == type) {
           out <- unique(partable$lhs[block.ind &
             partable$op == "=~" &
+            partable$lhs != partable$rhs & # no phantom
             !partable$lhs %in% rv.names])
           return.value$lv.regular[[b]] <- out
           next
@@ -255,14 +260,46 @@ lav_partable_vnames <- function(partable, type = NULL, ..., # nolint
           next
         }
 
-        # formative latent variables ONLY (ie defined by <~ only)
+        # formative latent variables: phantom lv + zero residual
         if ("lv.formative" == type) {
           out <- unique(partable$lhs[block.ind &
-            partable$op == "<~"])
+            partable$lhs == partable$rhs &
+            partable$op == "=~"])
+          rm.idx <- integer(0L)
+          for (i in seq_len(length(out))) {
+            var.idx <- which(block.ind &
+                             partable$op == "~~" &
+                             partable$lhs == out[i] &
+                             partable$lhs == partable$rhs)
+            if (length(var.idx) == 0L) {
+              next
+            }
+            if (!is.null(partable$mod.idx) && !is.null(partable$fixed)) {
+              if (partable$fixed[var.idx] != "0") {
+                rm.idx <- c(rm.idx, i)
+              }
+            } else if (!is.null(partable$free) && !is.null(partable$ustart)) {
+              if (partable$free[var.idx] > 0L ||
+                  partable$ustart[var.idx] != 0) {
+                rm.idx <- c(rm.idx, i)
+              }
+            }
+          }
+          if (length(rm.idx) > 0L) {
+            out <- out[-rm.idx]
+          }
           return.value$lv.formative[[b]] <- out
           next
         }
 
+        # composites defined by "<~"
+        if ("lv.composite" == type) {
+          out <- unique(partable$lhs[block.ind &
+            partable$op == "<~"])
+          return.value$lv.composite[[b]] <- out
+          next
+        }
+
         # lv's involved in efa
         if (any(type == c("lv.efa", "ov.efa"))) {
           if (is.null(partable$efa)) {
@@ -297,11 +334,26 @@ lav_partable_vnames <- function(partable, type = NULL, ..., # nolint
         if ("lv.ind" == type) {
           out <- unique(partable$rhs[block.ind &
             partable$op == "=~" &
-            partable$rhs %in% lv.names])
+            partable$rhs %in% lv.names &
+            partable$lhs != partable$rhs]) # no phantom lv's
           return.value$lv.ind[[b]] <- out
           next
         }
 
+        # higher-order latent variables
+        if ("lv.ho" == type) {
+          out.ind <- unique(partable$rhs[block.ind &
+            partable$op == "=~" &
+            partable$rhs %in% lv.names &
+            partable$lhs != partable$rhs]) # no phantom lv's
+          out <- unique(partable$lhs[block.ind &
+            partable$op == "=~" &
+            partable$lhs %in% lv.names &
+            partable$rhs %in% out.ind])
+          return.value$lv.ho[[b]] <- out
+          next
+        }
+
         # eqs.y
         if (!(any(type == c("lv", "lv.regular")))) {
           eqs.y <- unique(partable$lhs[block.ind &
@@ -333,20 +385,28 @@ lav_partable_vnames <- function(partable, type = NULL, ..., # nolint
             partable$op == "=~"])
         }
 
+        # v.cind -- indicators of composites
+        if (!(any(type == c("lv", "lv.regular")))) {
+          v.cind <- unique(partable$rhs[block.ind &
+            partable$op == "<~"])
+        }
+
         # ov.*
         if (!(any(type == c("lv", "lv.regular", "lv.x", "lv.y")))) {
           # 1. indicators, which are not latent variables themselves
           ov.ind <- v.ind[!v.ind %in% lv.names2]
+          # 1b. indicator of composites
+          ov.cind <- v.cind[!v.cind %in% lv.names2]
           # 2. dependent ov's
-          ov.y <- eqs.y[!eqs.y %in% c(lv.names2, ov.ind)]
+          ov.y <- eqs.y[!eqs.y %in% c(lv.names2, ov.ind, ov.cind)]
           # 3. independent ov's
           if (lav_partable_nlevels(partable) > 1L && b > 1L) {
             # NEW in 0.6-8: if an 'x' was an 'y' in a previous level,
             #               treat it as 'y'
             tmp.eqs.y <- unique(partable$lhs[partable$op == "~"]) # all blocks
-            ov.x <- eqs.x[!eqs.x %in% c(lv.names2, ov.ind, tmp.eqs.y)]
+            ov.x <- eqs.x[!eqs.x %in% c(lv.names2, ov.ind, ov.cind, tmp.eqs.y)]
           } else {
-            ov.x <- eqs.x[!eqs.x %in% c(lv.names2, ov.ind, ov.y)]
+            ov.x <- eqs.x[!eqs.x %in% c(lv.names2, ov.ind, ov.cind, ov.y)]
           }
           # new in 0.6-12: if we have interaction terms in ov.x, check
           # if some terms are in eqs.y; if so, remove the interaction term
@@ -386,7 +446,7 @@ lav_partable_vnames <- function(partable, type = NULL, ..., # nolint
               partable$op == "|") &
             !partable$lhs %in% lv.names2]
 
-          ov.tmp <- c(ov.ind, ov.y, ov.x)
+          ov.tmp <- c(ov.ind, ov.cind, ov.y, ov.x)
           ov.extra <- unique(c(ov.cov, ov.int)) # must be in this order!
           # so that
           # lav_partable_independence
@@ -405,6 +465,11 @@ lav_partable_vnames <- function(partable, type = NULL, ..., # nolint
           next
         }
 
+        if ("ov.cind" == type) {
+          return.value$ov.cind[[b]] <- ov.cind
+          next
+        }
+
         if ("ov.interaction" == type) {
           ov.int.names <- ov.names[grepl(":", ov.names, fixed = TRUE)]
           n.int <- length(ov.int.names)
@@ -570,7 +635,8 @@ lav_partable_vnames <- function(partable, type = NULL, ..., # nolint
         if ("lv.nonnormal" == type) {
           # regular lv's
           lv.reg <- unique(partable$lhs[block.ind &
-            partable$op == "=~"])
+                           partable$op == "=~" &
+                           partable$lhs != partable$rhs])
           if (length(lv.reg) > 0L) {
             out <- unlist(lapply(lv.reg, function(x) {
               # get indicators for this lv
@@ -756,6 +822,7 @@ lav_partable_vnames <- function(partable, type = NULL, ..., # nolint
         if (any("lv.regular" == type)) {
           out <- unique(partable$lhs[block.ind &
             partable$op == "=~" &
+            partable$lhs != partable$rhs & # no phantom
             !partable$lhs %in% rv.names])
           return.value$lv.regular[[b]] <- out
         }
@@ -766,13 +833,44 @@ lav_partable_vnames <- function(partable, type = NULL, ..., # nolint
           return.value$lv.interaction[[b]] <- lv.interaction
         }
 
-        # formative latent variables ONLY (ie defined by <~ only)
+        # formative latent variables: phantom lv + zero residual
         if (any("lv.formative" == type)) {
           out <- unique(partable$lhs[block.ind &
-            partable$op == "<~"])
+            partable$lhs == partable$rhs &
+            partable$op == "=~"])
+          rm.idx <- integer(0L)
+          for (i in seq_len(length(out))) {
+            var.idx <- which(block.ind &
+                             partable$op == "~~" &
+                             partable$lhs == out[i] &
+                             partable$lhs == partable$rhs)
+            if (length(var.idx) == 0L) {
+              next
+            }
+            if (!is.null(partable$mod.idx) && !is.null(partable$fixed)) {
+              if (partable$fixed[var.idx] != "0") {
+                rm.idx <- c(rm.idx, i)
+              }
+            } else if (!is.null(partable$free) && !is.null(partable$ustart)) {
+              if (partable$free[var.idx] > 0L ||
+                  partable$ustart[var.idx] != 0) {
+                rm.idx <- c(rm.idx, i)
+              }
+            }
+          }
+          if (length(rm.idx) > 0L) {
+            out <- out[-rm.idx]
+          }
           return.value$lv.formative[[b]] <- out
         }
 
+        # composite latent variables (ie defined by <~)
+        if (any("lv.composite" == type)) {
+          out <- unique(partable$lhs[block.ind &
+            partable$op == "<~"])
+          return.value$lv.composite[[b]] <- out
+        }
+
         # lv's involved in efa
         if (any(type %in% c("lv.efa", "ov.efa"))) {
           if (is.null(partable$efa)) {
@@ -803,10 +901,24 @@ lav_partable_vnames <- function(partable, type = NULL, ..., # nolint
         if (any("lv.ind" == type)) {
           out <- unique(partable$rhs[block.ind &
             partable$op == "=~" &
-            partable$rhs %in% lv.names])
+            partable$rhs %in% lv.names &
+            partable$lhs != partable$rhs]) # no phantom lv's
           return.value$lv.ind[[b]] <- out
         }
 
+        # higher-order latent variables
+        if (any("lv.ho" == type)) {
+          out.ind <- unique(partable$rhs[block.ind &
+            partable$op == "=~" &
+            partable$rhs %in% lv.names &
+            partable$lhs != partable$rhs]) # no phantom lv's
+          out <- unique(partable$lhs[block.ind &
+            partable$op == "=~" &
+            partable$lhs %in% lv.names &
+            partable$rhs %in% out.ind])
+          return.value$lv.ho[[b]] <- out
+        }
+
         # eqs.y
         eqs.y <- unique(partable$lhs[block.ind &
           partable$op == "~"])
@@ -830,19 +942,26 @@ lav_partable_vnames <- function(partable, type = NULL, ..., # nolint
         v.ind <- unique(partable$rhs[block.ind &
           partable$op == "=~"])
 
+        # v.cind -- indicators of composites
+        v.cind <- unique(partable$rhs[block.ind &
+          partable$op == "<~"])
+
+
         # ov.*
         # 1. indicators, which are not latent variables themselves
         ov.ind <- v.ind[!v.ind %in% lv.names2]
+        # 1a. indicators of composites
+        ov.cind <- v.cind[!v.cind %in% lv.names2]
         # 2. dependent ov's
-        ov.y <- eqs.y[!eqs.y %in% c(lv.names2, ov.ind)]
+        ov.y <- eqs.y[!eqs.y %in% c(lv.names2, ov.ind, ov.cind)]
         # 3. independent ov's
         if (lav_partable_nlevels(partable) > 1L && b > 1L) {
           # NEW in 0.6-8: if an 'x' was an 'y' in a previous level,
           #               treat it as 'y'
           tmp.eqs.y <- unique(partable$lhs[partable$op == "~"]) # all blocks
-          ov.x <- eqs.x[!eqs.x %in% c(lv.names2, ov.ind, tmp.eqs.y)]
+          ov.x <- eqs.x[!eqs.x %in% c(lv.names2, ov.ind, ov.cind, tmp.eqs.y)]
         } else {
-          ov.x <- eqs.x[!eqs.x %in% c(lv.names2, ov.ind, ov.y)]
+          ov.x <- eqs.x[!eqs.x %in% c(lv.names2, ov.ind, ov.cind, ov.y)]
         }
         # new in 0.6-12: if we have interaction terms in ov.x, check
         # if some terms are in eqs.y; if so, remove the interaction term
@@ -881,7 +1000,7 @@ lav_partable_vnames <- function(partable, type = NULL, ..., # nolint
             partable$op == "|") &
           !partable$lhs %in% lv.names2]
 
-        ov.tmp <- c(ov.ind, ov.y, ov.x)
+        ov.tmp <- c(ov.ind, ov.cind, ov.y, ov.x)
         ov.extra <- unique(c(ov.cov, ov.int)) # must be in this order!
         # so that
         # lav_partable_independence
@@ -897,6 +1016,10 @@ lav_partable_vnames <- function(partable, type = NULL, ..., # nolint
           return.value$ov.ind[[b]] <- ov.ind
         }
 
+        if (any("ov.cind" == type)) {
+          return.value$ov.cind[[b]] <- ov.cind
+        }
+
         if (any("ov.interaction" == type)) {
           ov.int.names <- ov.names[grepl(":", ov.names, fixed = TRUE)]
           n.int <- length(ov.int.names)
@@ -1054,7 +1177,8 @@ lav_partable_vnames <- function(partable, type = NULL, ..., # nolint
         if (any("lv.nonnormal" == type)) {
           # regular lv's
           lv.reg <- unique(partable$lhs[block.ind &
-            partable$op == "=~"])
+                           partable$op == "=~" &
+                           partable$lhs != partable$rhs])
           if (length(lv.reg) > 0L) {
             out <- unlist(lapply(lv.reg, function(x) {
               # get indicators for this lv
