@@ -9,14 +9,24 @@
 #                    categorical case
 # - YR 12 Jan 2014: refactoring + lav_predict_fy (to be used by estimator MML)
 #
+# - YR 25 Mar 2025: transformed factor scores (correlation-preserving)
 
 # overload standard R function `predict'
 setMethod(
   "predict", "lavaan",
-  function(object, newdata = NULL) {
+  function(object, newdata = NULL, ...) {
+    dotdotdot <- list(...)
+    if (length(dotdotdot) > 0L) {
+      for (j in seq_along(dotdotdot)) {
+        lav_msg_warn(gettextf(
+          "Unknown argument %s for %s", sQuote(names(dotdotdot)[j]),
+          sQuote("predict"))
+        )
+      }
+    }
     lavPredict(
       object = object, newdata = newdata, type = "lv", method = "EBM",
-      fsm = FALSE, optim.method = "bfgs"
+      fsm = FALSE, rel = FALSE, optim.method = "bfgs"
     )
   }
 )
@@ -41,7 +51,7 @@ predict.efaList <- function(object, ...) {
 lavPredict <- function(object, newdata = NULL, # keep order of predict(), 0.6-7
                        type = "lv", method = "EBM", transform = FALSE,
                        se = "none", acov = "none", label = TRUE, fsm = FALSE,
-                       mdist = FALSE,
+                       mdist = FALSE, rel = FALSE,
                        append.data = FALSE, assemble = FALSE, # or TRUE?
                        level = 1L, optim.method = "bfgs", ETA = NULL,
                        drop.list.single.group = TRUE) {
@@ -73,12 +83,21 @@ lavPredict <- function(object, newdata = NULL, # keep order of predict(), 0.6-7
     )
   }
   lavimplied <- object@implied
+  lavpartable <- object@ParTable
+
+  # warn if the model does not contain any 'regular' latent variables
+  if (length(lavNames(lavpartable, "lv.regular")) == 0L) {
+    lav_msg_warn(gettextf("fitted model does not contain regular
+    (i.e., measured) latent variables; the matrix of factor scores may
+    contain no columns"))
+  }
 
   res <- lav_predict_internal(
     lavmodel = lavmodel, lavdata = lavdata,
     lavsamplestats = lavsamplestats, lavimplied = lavimplied, lavh1 = lavh1,
-    lavpartable = object@ParTable, newdata = newdata, type = type, method = method,
-    transform = transform, se = se, acov = acov, label = label, fsm = fsm,
+    lavpartable = lavpartable, newdata = newdata, type = type, method = method,
+    transform = transform, se = se, acov = acov, label = label,
+    fsm = fsm, rel = rel,
     mdist = mdist, append.data = append.data, assemble = assemble,
     level = level, optim.method = optim.method, ETA = ETA,
     drop.list.single.group = drop.list.single.group
@@ -97,7 +116,8 @@ lav_predict_internal <- function(lavmodel = NULL,
                                  # standard options
                                  newdata = NULL, # keep order of predict(), 0.6-7
                                  type = "lv", method = "EBM", transform = FALSE,
-                                 se = "none", acov = "none", label = TRUE, fsm = FALSE,
+                                 se = "none", acov = "none", label = TRUE,
+                                 fsm = FALSE, rel = FALSE,
                                  mdist = FALSE,
                                  append.data = FALSE, assemble = FALSE, # or TRUE?
                                  level = 1L, optim.method = "bfgs", ETA = NULL,
@@ -220,7 +240,7 @@ lav_predict_internal <- function(lavmodel = NULL,
       lavdata = lavdata, lavsamplestats = lavsamplestats,
       lavimplied = lavimplied, se = se, acov = acov, level = level,
       data.obs = data.obs, eXo = eXo, method = method,
-      fsm = fsm, optim.method = optim.method
+      fsm = fsm, rel = rel, optim.method = optim.method
     )
 
     # extract fsm here
@@ -228,6 +248,11 @@ lav_predict_internal <- function(lavmodel = NULL,
       FSM <- attr(out, "fsm")
     }
 
+	# extract rel here
+	if (rel) {
+	  REL <- attr(out, "rel")
+	}
+
     # extract se here
     if (se != "none") {
       SE <- attr(out, "se")
@@ -289,11 +314,34 @@ lav_predict_internal <- function(lavmodel = NULL,
     # new in 0.6-16
     # we assume the dummy lv's have already been removed
     if (transform) {
-      VETA <- computeVETA(lavmodel = lavmodel, remove.dummy.lv = TRUE)
+      # VETA <- computeVETA(lavmodel = lavmodel, remove.dummy.lv = TRUE)
       EETA <- computeEETA(
         lavmodel = lavmodel,
         lavsamplestats = lavsamplestats, remove.dummy.lv = TRUE
       )
+      # compute transformation matrix
+      if (tolower(method) %in% c("ebm", "regression")) {
+        tmat <- lav_predict_tmat_green(lavmodel = lavmodel,
+                                       lavimplied = lavimplied)
+      } else {
+        tmat <- lav_predict_tmat_det(lavmodel = lavmodel,
+                                     lavimplied = lavimplied)
+      }
+
+      # update FSM
+      if (fsm) {
+        FSM <- lapply(seq_len(lavdata@ngroups), function(g) {
+          # determine block
+          if (lavdata@nlevels == 1L) {
+            bb <- g
+          } else {
+            bb <- (g - 1) * lavdata@nlevels + level
+          }
+          ret <- tmat[[bb]] %*% FSM[[g]]
+          ret
+        })
+      }
+
       out <- lapply(seq_len(lavdata@ngroups), function(g) {
         # determine block
         if (lavdata@nlevels == 1L) {
@@ -302,25 +350,19 @@ lav_predict_internal <- function(lavmodel = NULL,
           bb <- (g - 1) * lavdata@nlevels + level
         }
 
-        FS.centered <- scale(out[[g]],
-          center = TRUE,
-          scale = FALSE
-        )
-        FS.cov <- crossprod(FS.centered) / nrow(FS.centered)
-        FS.cov.inv <- try(solve(FS.cov), silent = TRUE)
-        if (inherits(FS.cov.inv, "try-error")) {
-          lav_msg_warn(
-            gettext("could not invert (co)variance matrix of factor scores;
-                    returning original factor scores."))
-          return(out[[g]])
-        }
-        fs.inv.sqrt <- lav_matrix_symmetric_sqrt(FS.cov.inv)
-        veta.sqrt <- lav_matrix_symmetric_sqrt(VETA[[g]])
-        if (fsm) {
-          # change FSM
-          FSM[[g]] <<- veta.sqrt %*% fs.inv.sqrt %*% FSM[[g]]
-        }
-        tmp <- FS.centered %*% fs.inv.sqrt %*% veta.sqrt
+        FS.centered <- scale(out[[g]], center = TRUE, scale = FALSE)
+        #FS.cov <- crossprod(FS.centered) / nrow(FS.centered)
+        #FS.cov.inv <- try(solve(FS.cov), silent = TRUE)
+        #if (inherits(FS.cov.inv, "try-error")) {
+        #  lav_msg_warn(
+        #    gettext("could not invert (co)variance matrix of factor scores;
+        #            returning original factor scores."))
+        #  return(out[[g]])
+        #}
+        #fs.inv.sqrt <- lav_matrix_symmetric_sqrt(FS.cov.inv)
+        #veta.sqrt <- lav_matrix_symmetric_sqrt(VETA[[g]])
+        #tmp <- FS.centered %*% fs.inv.sqrt %*% veta.sqrt
+        tmp <- FS.centered %*% t(tmat[[bb]])
         ret <- t(t(tmp) + drop(EETA[[g]]))
 
         ret
@@ -450,6 +492,12 @@ lav_predict_internal <- function(lavmodel = NULL,
           }
         }
 
+        if (rel) {
+          if (!is.null(REL[[g]])) {
+            names(REL[[g]]) <- lavpta$vnames$lv[[gg]]
+          }
+        }
+
         if (acov != "none") {
           if (is.null(ACOV[[g]])) {
             # skip
@@ -627,6 +675,10 @@ lav_predict_internal <- function(lavmodel = NULL,
     attr(res, "fsm") <- FSM
   }
 
+  if (rel && type == "lv") {
+    attr(res, "rel") <- REL
+  }
+
   if (mdist) {
     attr(res, "mdist") <- MDIST
   }
@@ -653,6 +705,7 @@ lav_predict_eta <- function(lavobject = NULL, # for convenience
                             # options
                             method = "EBM",
                             fsm = FALSE,
+							rel = FALSE,
                             se = "none", acov = "none",
                             level = 1L,
                             optim.method = "bfgs") {
@@ -681,7 +734,7 @@ lav_predict_eta <- function(lavobject = NULL, # for convenience
         lavmodel = lavmodel, lavdata = lavdata,
         lavimplied = lavimplied, se = se, acov = acov,
         level = level, lavsamplestats = lavsamplestats,
-        data.obs = data.obs, eXo = eXo, fsm = fsm
+        data.obs = data.obs, eXo = eXo, fsm = fsm, rel = rel
       )
     } else if (method == "ml") {
       out <- lav_predict_eta_bartlett(
@@ -689,7 +742,7 @@ lav_predict_eta <- function(lavobject = NULL, # for convenience
         lavmodel = lavmodel, lavdata = lavdata,
         lavimplied = lavimplied, se = se, acov = acov,
         level = level, lavsamplestats = lavsamplestats,
-        data.obs = data.obs, eXo = eXo, fsm = fsm
+        data.obs = data.obs, eXo = eXo, fsm = fsm, rel = rel
       )
     } else {
       lav_msg_stop(gettextf("unkown method: %s.", method))
@@ -731,7 +784,7 @@ lav_predict_eta_normal <- function(lavobject = NULL, # for convenience
                                    # optional new data
                                    data.obs = NULL, eXo = NULL,
                                    se = "none", acov = "none", level = 1L,
-                                   fsm = FALSE) {
+                                   fsm = FALSE, rel = FALSE) {
   # full object?
   if (inherits(lavobject, "lavaan")) {
     lavmodel <- lavobject@Model
@@ -776,6 +829,9 @@ lav_predict_eta_normal <- function(lavobject = NULL, # for convenience
   if (fsm) {
     FSM <- vector("list", length = lavdata@ngroups)
   }
+  if (rel) {
+    REL <- vector("list", length = lavdata@ngroups)
+  }
 
   if (acov != "none") {
     se <- acov # ACOV implies SE
@@ -860,6 +916,16 @@ lav_predict_eta_normal <- function(lavobject = NULL, # for convenience
     # center data
     Yc <- t(t(data.obs.g) - EY.g)
 
+    # sampling weights? -- CHECKME: needed??
+    if (.hasSlot(lavdata, "weights") &&
+        !is.null(lavdata@weights[[g]]) && level == 1L) {
+      # EY.g is already weighted
+      # use sampling.weights.normalization == "group"
+      WT <- lavdata@weights[[g]]
+      WT2 <- WT / sum(WT) * lavdata@nobs[[g]]
+      Yc <- Yc * sqrt(WT2)
+    }
+
     # global factor score coefficient matrix 'C'
     FSC <- VETA.g %*% t(LAMBDA.g) %*% Sigma.inv.g
 
@@ -868,6 +934,20 @@ lav_predict_eta_normal <- function(lavobject = NULL, # for convenience
       FSM.g <- FSC
     }
 
+	# reliability?
+	if (rel) {
+	  Var.f <- FSC %*% Sigma.hat[[g]] %*% t(FSC)  # or S?
+	  Cov.f.eta <- FSC %*% LAMBDA.g %*% VETA.g
+	  Var.eta <- VETA.g
+	  # FS.determinacy <- diag( diag(1/sqrt(diag(Var.f))) %*%
+	  #                         Cov.f.eta %*%
+	  #                         diag(1/sqrt(diag(Var.eta)))
+	  #                       )
+	  FS.determinacy <- ( diag(Cov.f.eta) /
+	                      (sqrt(diag(Var.f)) * sqrt(diag(Var.eta))) )
+	  REL.g <- FS.determinacy*FS.determinacy
+	}
+
     # compute factor scores
     if (lavdata@missing %in% c("ml", "ml.x")) {
       # missing patterns for this group
@@ -961,6 +1041,11 @@ lav_predict_eta_normal <- function(lavobject = NULL, # for convenience
       FSM[[g]] <- FSM.g
     }
 
+	# REL
+	if (rel) {
+	  REL[[g]] <- REL.g
+	}
+
     # standard error
     if (se == "standard") {
       if (lavdata@missing %in% c("ml", "ml.x")) {
@@ -986,6 +1071,9 @@ lav_predict_eta_normal <- function(lavobject = NULL, # for convenience
   if (fsm) {
     attr(FS, "fsm") <- FSM
   }
+  if (rel) {
+    attr(FS, "rel") <- REL
+  }
   if (se != "none") {
     attr(FS, "se") <- SE
     # return full sampling covariance matrix?
@@ -1021,7 +1109,7 @@ lav_predict_eta_bartlett <- function(lavobject = NULL, # for convenience
                                      # optional new data
                                      data.obs = NULL, eXo = NULL,
                                      se = "none", acov = "none", level = 1L,
-                                     fsm = FALSE) {
+                                     fsm = FALSE, rel = FALSE) {
   # full object?
   if (inherits(lavobject, "lavaan")) {
     lavmodel <- lavobject@Model
@@ -1056,7 +1144,7 @@ lav_predict_eta_bartlett <- function(lavobject = NULL, # for convenience
   }
 
   LAMBDA <- computeLAMBDA(lavmodel = lavmodel, remove.dummy.lv = FALSE)
-  Sigma <- lavimplied$cov
+  Sigma.hat <- lavimplied$cov
   Sigma.inv <- lapply(lavimplied$cov, MASS::ginv)
   VETA <- computeVETA(lavmodel = lavmodel) # for se only
   EETA <- computeEETA(lavmodel = lavmodel, lavsamplestats = lavsamplestats)
@@ -1066,6 +1154,9 @@ lav_predict_eta_bartlett <- function(lavobject = NULL, # for convenience
   if (fsm) {
     FSM <- vector("list", length = lavdata@ngroups)
   }
+  if (rel) {
+    REL <- vector("list", length = lavdata@ngroups)
+  }
 
   if (acov != "none") se <- acov # ACOV implies SE
   if (se != "none") {
@@ -1150,6 +1241,16 @@ lav_predict_eta_bartlett <- function(lavobject = NULL, # for convenience
     # center data
     Yc <- t(t(data.obs.g) - EY.g)
 
+    # sampling weights? CHECKME: needed??
+    if (.hasSlot(lavdata, "weights") &&
+        !is.null(lavdata@weights[[g]]) && level == 1L) {
+      # EY.g is already weighted
+      # use sampling.weights.normalization == "group"
+      WT <- lavdata@weights[[g]]
+      WT2 <- WT / sum(WT) * lavdata@nobs[[g]]
+      Yc <- Yc * sqrt(WT2)
+    }
+
     # global factor score coefficient matrix 'C'
     FSC <- (MASS::ginv(t(LAMBDA.g) %*% Sigma.inv.g %*% LAMBDA.g)
     %*% t(LAMBDA.g) %*% Sigma.inv.g)
@@ -1160,6 +1261,20 @@ lav_predict_eta_bartlett <- function(lavobject = NULL, # for convenience
       FSM.g <- FSC
     }
 
+	# reliability?
+	if (rel) {
+      Var.f <- FSC %*% Sigma.hat[[g]] %*% t(FSC)  # or S?
+      Cov.f.eta <- FSC %*% LAMBDA.g %*% VETA.g
+      Var.eta <- VETA.g
+      # FS.determinacy <- diag( diag(1/sqrt(diag(Var.f))) %*%
+      #                         Cov.f.eta %*%
+      #                         diag(1/sqrt(diag(Var.eta)))
+      #                       )
+      FS.determinacy <- ( diag(Cov.f.eta) /
+                          (sqrt(diag(Var.f)) * sqrt(diag(Var.eta))) )
+      REL.g <- FS.determinacy*FS.determinacy
+	}
+
     # compute factor scores
     if (lavdata@missing %in% c("ml", "ml.x")) {
       # missing patterns for this group
@@ -1278,6 +1393,11 @@ lav_predict_eta_bartlett <- function(lavobject = NULL, # for convenience
       FSM[[g]] <- FSM.g
     }
 
+	# REL
+	if (rel) {
+	  REL[[g]] <- REL.g
+	}
+
     # standard error
     if (se == "standard") {
       if (lavdata@missing %in% c("ml", "ml.x")) {
@@ -1310,6 +1430,9 @@ lav_predict_eta_bartlett <- function(lavobject = NULL, # for convenience
   if (fsm) {
     attr(FS, "fsm") <- FSM
   }
+  if (rel) {
+    attr(FS, "rel") <- REL
+  }
   if (se != "none") {
     attr(FS, "se") <- SE
     # return full sampling covariance matrix?
@@ -1875,3 +1998,94 @@ lav_predict_fy_eta.i <- function(lavmodel = NULL, lavdata = NULL,
 
   FY
 }
+
+# compute `transformation' matrix to convert regression factor scores
+# to (Green's) correlation-preserving factor scores
+lav_predict_tmat_green <- function(lavobject = NULL,
+                                   lavmodel = NULL, lavimplied = NULL) {
+
+  if (!is.null(lavobject)) {
+    lavmodel <- lavobject@Model
+    lavimplied <- lavobject@implied
+  }
+
+  if (is.null(lavimplied) || length(lavimplied) == 0L) {
+    lavimplied <- lav_model_implied(lavmodel)
+  }
+  if (lavmodel@conditional.x) {
+    lavimplied <- lav_model_implied_cond2uncond(lavimplied)
+  }
+  Sigma <- lavimplied$cov
+  VETA <- computeVETA(lavmodel = lavmodel, remove.dummy.lv = TRUE)
+  LAMBDA <- computeLAMBDA(lavmodel)
+
+  nblocks <- lavmodel@nblocks
+  tmat <- vector("list", length = nblocks)
+
+  # compute tmat per block
+  for(b in seq_len(nblocks)) {
+    Sigma.b <- Sigma[[b]]
+    Sigma.b.inv <- solve(Sigma.b)
+    Veta <- VETA[[b]]
+    Veta.sqrt <- lav_matrix_symmetric_sqrt(Veta)
+    Veta32 <- Veta %*% Veta.sqrt
+    Lambda <- LAMBDA[[b]]
+    tmp <- Veta32 %*% t(Lambda) %*% Sigma.b.inv %*% Lambda %*% Veta32
+    tmp.inv.sqrt <- lav_matrix_symmetric_sqrt(solve(tmp))
+    tmat[[b]] <- Veta.sqrt %*% tmp.inv.sqrt %*% Veta.sqrt
+  }
+
+  tmat
+}
+
+# compute `transformation' matrix to convert Bartlett factor scores
+# to (Krijnen/McDonald) correlation-preserving factor scores
+lav_predict_tmat_det <- function(lavobject = NULL,
+                                 lavmodel = NULL, lavimplied = NULL) {
+
+  if (!is.null(lavobject)) {
+    lavmodel <- lavobject@Model
+    lavimplied <- lavobject@implied
+  }
+
+  if (is.null(lavimplied) || length(lavimplied) == 0L) {
+    lavimplied <- lav_model_implied(lavmodel)
+  }
+  if (lavmodel@conditional.x) {
+    lavimplied <- lav_model_implied_cond2uncond(lavimplied)
+  }
+  Sigma <- lavimplied$cov
+  VETA <- computeVETA(lavmodel = lavmodel, remove.dummy.lv = TRUE)
+  LAMBDA <- computeLAMBDA(lavmodel)
+
+  nblocks <- lavmodel@nblocks
+  tmat <- vector("list", length = nblocks)
+
+  # compute tmat per block
+  for(b in seq_len(nblocks)) {
+    Sigma.b <- Sigma[[b]]
+    Sigma.b.inv <- solve(Sigma.b)
+    Veta <- VETA[[b]]
+    Veta.sqrt <- lav_matrix_symmetric_sqrt(Veta)
+    Veta.inv.sqrt <- lav_matrix_symmetric_sqrt(solve(Veta))
+    Lambda <- LAMBDA[[b]]
+    tmp <- Veta.sqrt %*% t(Lambda) %*% Sigma.b.inv %*% Lambda %*% Veta.sqrt
+    tmp.sqrt <- lav_matrix_symmetric_sqrt(tmp)
+    tmat[[b]] <- Veta.sqrt %*% tmp.sqrt %*% Veta.inv.sqrt
+  }
+
+  tmat
+}
+
+# single block only, for internal use in lav_sam_step1_local()
+lav_predict_tmat_det_internal <- function(Sigma = NULL, Veta = NULL,
+                                          Lambda = NULL) {
+    Sigma.inv <- solve(Sigma)
+    Veta.sqrt <- lav_matrix_symmetric_sqrt(Veta)
+    Veta.inv.sqrt <- lav_matrix_symmetric_sqrt(solve(Veta))
+    tmp <- Veta.sqrt %*% t(Lambda) %*% Sigma.inv %*% Lambda %*% Veta.sqrt
+    tmp.sqrt <- lav_matrix_symmetric_sqrt(tmp)
+    tmat <- Veta.sqrt %*% tmp.sqrt %*% Veta.inv.sqrt
+    tmat
+}
+
