@@ -210,9 +210,8 @@ ldw_parse_step1 <- function(modelsrc, types) {
       elem.i <- elem.i + 1L
     }
   }
-  identifiers <- gregexpr(
-    "[ \n][_.a-zA-Z][._a-zA-Z0-9]*",
-    paste0(" ", modelsrcw)
+  identifiers <- gregexpr("[ \n][_.[:alpha:]][_.[:alnum:]]*",
+                 paste0(" ", modelsrcw)
   )[[1]]
   identifier.lengths <- attr(identifiers, "match.length") - 1L
   for (i in seq_along(identifiers)) {
@@ -280,7 +279,7 @@ ldw_parse_step1 <- function(modelsrc, types) {
     if (elem.type[i] == types$identifier && elem.text[i] == "efa") {
       frm.hasefa <- TRUE
     }
-    if (any(elem.text[i] == c("+", "*", "=~"))) {
+    if (any(elem.text[i] == c("+", "*", "=~", "-"))) {
       if (frm.incremented) {
         frm.number <- frm.number - 1L
         elem.formula.number[i] <- frm.number
@@ -593,7 +592,7 @@ ldw_parse_get_modifier <- function(formul1, lhs, opi, modelsrc, types,
         if (j + 3L < nelem && formul1$elem.text[j + 3L] == "," &&
           any(formul1$elem.text[j] == c(
             "start", "fixed", "label",
-            "upp", "lower", "rv", "prior"
+            "upper", "lower", "rv", "prior"
           ))) {
           vector.type <- 0
           labnu <- j + 2L
@@ -815,9 +814,14 @@ ldw_parse_model_string <- function(model.syntax = "", as.data.frame. = FALSE) {
     }
     nelem <- length(formul1$elem.type)
     # where is the operator
-    opi <- match(types$lavaanoperator, formul1$elem.type)
-    # opi <- which(formul1$elem.type == types$lavaanoperator)
-    # if (length(opi) > 1L) opi <- opi[1L]
+    opi <- which(formul1$elem.type == types$lavaanoperator)
+    if (length(opi) > 1L) { # if more then 1 operator skip operators ':'
+      opii <- 1L
+      while (formul1$elem.text[opi[opii]] == ":" && opii < length(opi)) {
+        opii <- opii + 1L
+      }
+      opi <- opi[opii]
+    }
     op <- formul1$elem.text[opi]
     if (any(op == constraint_operators)) { # ----- constraints -------
       lhs <- paste(formul1$elem.text[seq.int(1L, opi - 1L)], collapse = "")
@@ -954,7 +958,8 @@ ldw_parse_model_string <- function(model.syntax = "", as.data.frame. = FALSE) {
     colons <- which(formul1$elem.text[seq.int(1L, nelem - 1L)] == ":" &
       formul1$elem.type[seq.int(2L, nelem)] == types$identifier)
     # check at most 1 colon
-    if (length(colons) > 1) {
+    if (length(colons) > 2L ||
+        (length(colons) == 2L && (colons[1L] > opi || colons[2L] < opi))) {
       tl <- ldw_txtloc(modelsrc, formul1$elem.pos[colons[2]])
       lav_msg_stop(
         gettext(
@@ -966,13 +971,28 @@ ldw_parse_model_string <- function(model.syntax = "", as.data.frame. = FALSE) {
         ), tl[1L], footer = tl[2L]
       )
     }
-    if (length(colons) == 1) {
+    if (length(colons) > 0L) {
       # collapse items around colon "a" ":" "b" => "a:b"
-      formul1$elem.text[colons - 1L] <-
-        paste(formul1$elem.text[seq.int(colons - 1L, colons + 1L)],
+      formul1$elem.text[colons[1L] - 1L] <-
+        paste(formul1$elem.text[seq.int(colons[1L] - 1L, colons[1L] + 1L)],
           collapse = ""
         )
-      formul1 <- ldw_parse_sublist(formul1, seq.int(1L, colons - 1L))
+      formul1 <- ldw_parse_sublist(formul1,
+          setdiff(seq.int(1L, nelem), seq.int(colons[1L], colons[1L] + 1L)))
+      nelem <- length(formul1$elem.type)
+      if (colons[1L] < opi) {
+        opi <- opi - 2L # is in LHS
+        if (length(colons) == 2L) colons[2L] <- colons[2L] - 2L
+      }
+    }
+    if (length(colons) == 2L) {
+      # collapse items around colon "a" ":" "b" => "a:b"
+      formul1$elem.text[colons[2L] - 1L] <-
+        paste(formul1$elem.text[seq.int(colons[2L] - 1L, colons[2L] + 1L)],
+              collapse = ""
+        )
+      formul1 <- ldw_parse_sublist(formul1,
+          setdiff(seq.int(1L, nelem), seq.int(colons[2L], colons[2L] + 1L)))
       nelem <- length(formul1$elem.type)
     }
     # modifiers
@@ -1093,6 +1113,18 @@ ldw_parse_model_string <- function(model.syntax = "", as.data.frame. = FALSE) {
         }
       }
     }
+    # check for variable regressed on itself
+    if (formul1$elem.text[opi] == "~" &&
+        formul1$elem.text[opi - 1L] == formul1$elem.text[nelem]) {
+      if (!grepl("^0\\.?0*$", flat.fixed[idx])) {
+        tl <- ldw_txtloc(modelsrc, formul1$elem.pos[opi])
+        lav_msg_stop(
+          gettext("a variable cannot be regressed on itself"),
+          tl[1L],
+          footer = tl[2L]
+        )
+      }
+    }
   }
   # create flat (omit items without operator)
   filled.ones <- which(flat.op != "")
@@ -1116,6 +1148,48 @@ ldw_parse_model_string <- function(model.syntax = "", as.data.frame. = FALSE) {
   if (length(int.idx) > 0L) {
     flat$op[int.idx] <- "~1"
   }
+  # if there are constraints that are simple lower or upper limits, put
+  # them in these members, add a modifier and remove constraint
+  aantal <- length(constraints)
+  if (aantal > 0) {
+    for (j in aantal:1) {
+      if (any(flat$label == constraints[[j]]$lhs) &&
+          any(constraints[[j]]$op == c("<", ">"))) {
+          rhslang <- str2lang(constraints[[j]]$rhs)
+          numbound <- NA_real_
+          if (mode(rhslang) == "numeric") {
+            numbound <- as.numeric(constraints[[j]]$rhs)
+          } else {
+            if (mode(rhslang) == "call") {
+              if (is.numeric(tryCatch(eval(rhslang),
+                                      error = function(e) "error"))) {
+                numbound <- eval(rhslang)
+              }
+            }
+          }
+          if (!is.na(numbound)) {
+            nrs <- which(flat$label == constraints[[j]]$lhs)
+            for (nr in nrs) {
+              nrm <- length(mod) + 1L
+              if (flat$mod.idx[nr] > 0L) {
+                nrm <- flat$mod.idx[nr]
+              } else {
+                flat$mod.idx[nr] <- nrm
+                mod <- c(mod, list(label = constraints[[j]]$lhs))
+              }
+              if (constraints[[j]]$op == "<") {
+                flat$upper[nr] <- as.character(numbound)
+                mod[[nrm]]$upper <- numbound
+              } else {
+                flat$lower[nr] <- as.character(numbound)
+                mod[[nrm]]$lower <- numbound
+              }
+            }
+            constraints <- constraints[-j]
+          }
+        }
+    }
+  }
   # new in 0.6, reorder covariances here!
   flat <- lav_partable_covariance_reorder(flat)
   if (as.data.frame.) {
@@ -1135,6 +1209,7 @@ ldw_parse_model_string <- function(model.syntax = "", as.data.frame. = FALSE) {
       }
     }
   }
+  # create output
   attr(flat, "modifiers") <- mod
   attr(flat, "constraints") <- constraints
   flat
