@@ -281,7 +281,7 @@ lav_model_test <- function(lavobject = NULL,
   # handle equality constraints (note: we ignore inequality constraints,
   # active or not!)
   # we use the rank of con.jac (even if the constraints are nonlinear)
-  if (nrow(lavmodel@con.jac) > 0L) {
+  if (!lavmodel@cin.simple.only && nrow(lavmodel@con.jac) > 0L) {
     ceq.idx <- attr(lavmodel@con.jac, "ceq.idx")
     if (length(ceq.idx) > 0L) {
       neq <- qr(lavmodel@con.jac[ceq.idx, , drop = FALSE])$rank
@@ -399,6 +399,12 @@ lav_model_test <- function(lavobject = NULL,
     } else {
       chisq.group <- rep(as.numeric(NA), lavdata@ngroups)
     }
+  } else if (lavdata@missing == "ml" && any(lavdata@Mp[[1]]$coverage == 0)) {
+    if (length(lavh1) > 0L) {
+      chisq.group <- -2 * (lavloglik$loglik.group - lavh1$logl$loglik.group)
+    } else {
+      chisq.group <- rep(as.numeric(NA), lavdata@ngroups)
+    }
   } else {
     # get fx.group
     fx <- attr(x, "fx")
@@ -715,8 +721,8 @@ lav_update_test_custom_h1 <- function(lav_obj_h0, lav_obj_h1) {
   newTEST <- lav_obj_h0@test
 
   ## assemble a call to lavTestLRT()
-  lrtCallTemplate <- list(quote(lavTestLRT), object = quote(lav_obj_h1),
-                          quote(lav_obj_h0)) # in ...
+  lrtCallTemplate <- list(quote(lavTestLRT), object = quote(lav_obj_h0),
+                          quote(lav_obj_h1)) # in ...
 
   ## can only update tests available in both objects
   testNames0 <- names(lav_obj_h0@test)
@@ -769,7 +775,7 @@ lav_update_test_custom_h1 <- function(lav_obj_h0, lav_obj_h1) {
       newTEST[[tn]]$shift.parameter <- attr(ANOVA, "shift")[2] # first row is NA
     } else {
       ## unless scaled.shifted, RMSEA is calculated from $standard$stat and
-      ## df == sum($trace.UGamma).  Reverse-engineer from $scaling factor:
+      ## df == sum($trace.UGamma).  Reverse-engineer from $scaling.factor:
       newTEST[[tn]]$trace.UGamma <- newTEST[[tn]]$df * newTEST[[tn]]$scaling.factor
     }
     ## should not be necessary to replace $trace.UGamma2
